@@ -1,53 +1,101 @@
+import copy
+import math
+import os
 from concurrent.futures import ThreadPoolExecutor
+from contextlib import contextmanager
 from datetime import datetime
 from decimal import Decimal
-from typing import List
+from operator import itemgetter
+from pathlib import Path
+from tempfile import NamedTemporaryFile
+from typing import List, Optional
 
-import copy
+import cv2
 import geojson
 import numpy as np
-import math
-from fastlabel.const import AnnotationType
-import os
+import requests
 
+from fastlabel.const import AnnotationType
 from fastlabel.exceptions import FastLabelInvalidException
+from fastlabel.utils import is_video_project_type
 
 # COCO
 
 
-def to_coco(tasks: list, annotations: list = []) -> dict:
+def to_coco(
+    project_type: str, tasks: list, output_dir: str, annotations: list = []
+) -> dict:
     # Get categories
-    categories = __get_categories(tasks, annotations)
+    categories = __get_coco_categories(tasks, annotations)
 
     # Get images and annotations
     images = []
     annotations = []
     annotation_id = 0
-    image_id = 0
+    image_index = 0
     for task in tasks:
         if task["height"] == 0 or task["width"] == 0:
             continue
 
-        image_id += 1
-        image = {
-            "file_name": task["name"],
-            "height": task["height"],
-            "width": task["width"],
-            "id": image_id,
-        }
-        images.append(image)
+        if is_video_project_type(project_type):
+            image_file_names = _export_image_files_for_video_task(
+                task, str((Path(output_dir) / "images").resolve())
+            )
+            task_images = _generate_coco_images(
+                image_file_names=image_file_names,
+                height=task["height"],
+                width=task["width"],
+                offset=image_index,
+            )
+            image_index = len(task_images) + image_index
 
-        data = [{"annotation": annotation, "categories": categories,
-                 "image": image} for annotation in task["annotations"]]
-        with ThreadPoolExecutor(max_workers=8) as executor:
-            results = executor.map(__to_annotation, data)
+            def get_annotation_points(anno, index):
+                return _get_annotation_points_for_video_annotation(anno, index)
 
-        for result in results:
-            annotation_id += 1
-            if not result:
+        else:
+            image_index += 1
+            task_images = [
+                {
+                    "file_name": task["name"],
+                    "height": task["height"],
+                    "width": task["width"],
+                    "id": image_index,
+                }
+            ]
+
+            def get_annotation_points(anno, _):
+                return _get_annotation_points_for_image_annotation(anno)
+
+        for index, task_image in enumerate(task_images, 1):
+            images.append(task_image)
+            params = [
+                {
+                    "annotation_value": annotation["value"],
+                    "annotation_type": annotation["type"],
+                    "annotation_points": get_annotation_points(annotation, index),
+                    "annotation_keypoints": annotation.get("keypoints"),
+                    "categories": categories,
+                    "image_id": task_image["id"],
+                }
+                for annotation in task["annotations"]
+            ]
+
+            with ThreadPoolExecutor(max_workers=8) as executor:
+                image_annotations = executor.map(__to_coco_annotation, params)
+
+            filtered_image_annotations = list(filter(None, image_annotations))
+            if len(filtered_image_annotations) <= 0:
                 continue
-            result["id"] = annotation_id
-            annotations.append(result)
+
+            for image_annotation in sorted(
+                filtered_image_annotations,
+                key=itemgetter("image_id", "category_id", "area"),
+            ):
+                annotation_id += 1
+                if not image_annotation:
+                    continue
+                image_annotation["id"] = annotation_id
+                annotations.append(image_annotation)
 
     return {
         "images": images,
@@ -56,6 +104,20 @@ def to_coco(tasks: list, annotations: list = []) -> dict:
     }
 
 
+def _generate_coco_images(
+    image_file_names: str, height: int, width: int, offset: int = 0
+):
+    return [
+        {
+            "file_name": file_name,
+            "height": height,
+            "width": width,
+            "id": (index + 1) + offset,
+        }
+        for index, file_name in enumerate(image_file_names)
+    ]
+
+
 def __get_coco_skeleton(keypoints: list) -> list:
     keypoint_id_skeleton_index_map = {}
     for index, keypoint in enumerate(keypoints, 1):
@@ -69,21 +131,26 @@ def __get_coco_skeleton(keypoints: list) -> list:
         edges = keypoint["edges"]
         for edge in edges:
             edge_skeleton_index = keypoint_id_skeleton_index_map[edge]
-            if not edge_skeleton_index in filtered_skeleton_indexes:
+            if edge_skeleton_index not in filtered_skeleton_indexes:
                 skeleton.append([skeleton_index, edge_skeleton_index])
             filtered_skeleton_indexes.append(skeleton_index)
     return skeleton
 
 
-def __get_categories(tasks: list, annotations: list) -> list:
+def __get_coco_categories(tasks: list, annotations: list) -> list:
     categories = []
     values = []
     for task in tasks:
         for task_annotation in task["annotations"]:
-            if task_annotation["type"] not in [AnnotationType.bbox.value, AnnotationType.polygon.value, AnnotationType.segmentation.value, AnnotationType.pose_estimation.value]:
+            if task_annotation["type"] not in [
+                AnnotationType.bbox.value,
+                AnnotationType.polygon.value,
+                AnnotationType.segmentation.value,
+                AnnotationType.pose_estimation.value,
+            ]:
                 continue
             values.append(task_annotation["value"])
-    values = list(set(values))
+    values = sorted(list(set(values)))
 
     # Create categories from task annotations (not support pose esitimation)
     if not annotations:
@@ -92,10 +159,11 @@ def __get_categories(tasks: list, annotations: list) -> list:
                 "skeleton": [],
                 "keypoints": [],
                 "keypoint_colors": [],
+                # BUG: All are set to the same color.
                 "color": task_annotation["color"],
                 "supercategory": value,
                 "id": index,
-                "name": value
+                "name": value,
             }
             categories.append(category)
         return categories
@@ -114,44 +182,61 @@ def __get_categories(tasks: list, annotations: list) -> list:
                 coco_keypoints.append(keypoint["key"])
                 coco_keypoint_colors.append(keypoint["color"])
             coco_skeleton = __get_coco_skeleton(keypoints)
-        category = {"skeleton": coco_skeleton,
-                    "keypoints": coco_keypoints,
-                    "keypoint_colors": coco_keypoint_colors,
-                    "color": annotation["color"],
-                    "supercategory": annotation["value"],
-                    "id": index,
-                    "name": annotation["value"]}
+        category = {
+            "skeleton": coco_skeleton,
+            "keypoints": coco_keypoints,
+            "keypoint_colors": coco_keypoint_colors,
+            "color": annotation["color"],
+            "supercategory": annotation["value"],
+            "id": index,
+            "name": annotation["value"],
+        }
         index += 1
         categories.append(category)
     return categories
 
 
-def __to_annotation(data: dict) -> dict:
-    annotation = data["annotation"]
+def __to_coco_annotation(data: dict) -> dict:
     categories = data["categories"]
-    image = data["image"]
-    points = annotation.get("points")
-    keypoints = annotation.get("keypoints")
-    annotation_type = annotation["type"]
+    image_id = data["image_id"]
+    points = data["annotation_points"]
+    keypoints = data["annotation_keypoints"]
+    annotation_type = data["annotation_type"]
+    annotation_value = data["annotation_value"]
     annotation_id = 0
 
-    if annotation_type not in [AnnotationType.bbox.value, AnnotationType.polygon.value, AnnotationType.segmentation.value, AnnotationType.pose_estimation.value]:
+    if annotation_type not in [
+        AnnotationType.bbox.value,
+        AnnotationType.polygon.value,
+        AnnotationType.segmentation.value,
+        AnnotationType.pose_estimation.value,
+    ]:
         return None
-    if annotation_type != AnnotationType.pose_estimation.value and (not points or len(points)) == 0:
+    if annotation_type != AnnotationType.pose_estimation.value and (
+        not points or (len(points) == 0)
+    ):
         return None
-    if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
+    if annotation_type == AnnotationType.bbox.value and (
+        int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])
+    ):
         return None
 
-    category = __get_category_by_name(categories, annotation["value"])
+    category = __get_coco_category_by_name(categories, annotation_value)
+    if category is None:
+        return None
 
-    return __get_annotation(
-        annotation_id, points, keypoints, category["id"], image, annotation_type)
+    return __get_coco_annotation(
+        annotation_id, points, keypoints, category["id"], image_id, annotation_type
+    )
 
 
-def __get_category_by_name(categories: list, name: str) -> str:
-    category = [
-        category for category in categories if category["name"] == name][0]
-    return category
+def __get_coco_category_by_name(categories: list, name: str) -> Optional[dict]:
+    matched_categories = [
+        category for category in categories if category["name"] == name
+    ]
+    if len(matched_categories) >= 1:
+        return matched_categories[0]
+    return None
 
 
 def __get_coco_annotation_keypoints(keypoints: list) -> list:
@@ -167,16 +252,23 @@ def __get_coco_annotation_keypoints(keypoints: list) -> list:
     return coco_annotation_keypoints
 
 
-def __get_annotation(id_: int, points: list, keypoints: list, category_id: int, image: dict, annotation_type: str) -> dict:
+def __get_coco_annotation(
+    id_: int,
+    points: list,
+    keypoints: list,
+    category_id: int,
+    image_id: str,
+    annotation_type: str,
+) -> dict:
     annotation = {}
     annotation["num_keypoints"] = len(keypoints) if keypoints else 0
-    annotation["keypoints"] = __get_coco_annotation_keypoints(
-        keypoints) if keypoints else []
-    annotation["segmentation"] = __to_coco_segmentation(
-        annotation_type, points)
+    annotation["keypoints"] = (
+        __get_coco_annotation_keypoints(keypoints) if keypoints else []
+    )
+    annotation["segmentation"] = __to_coco_segmentation(annotation_type, points)
     annotation["iscrowd"] = 0
     annotation["area"] = __to_area(annotation_type, points)
-    annotation["image_id"] = image["id"]
+    annotation["image_id"] = image_id
     annotation["bbox"] = __to_bbox(annotation_type, points)
     annotation["category_id"] = category_id
     annotation["id"] = id_
@@ -205,7 +297,7 @@ def __to_bbox(annotation_type: str, points: list) -> list:
     else:
         base_points = points
     points_splitted = [
-        base_points[idx: idx + 2] for idx in range(0, len(base_points), 2)
+        base_points[idx : idx + 2] for idx in range(0, len(base_points), 2)
     ]
     polygon_geo = geojson.Polygon(points_splitted)
     coords = np.array(list(geojson.utils.coords(polygon_geo)))
@@ -237,12 +329,13 @@ def __calc_area(annotation_type: str, points: list) -> float:
         width = points[0] - points[2]
         height = points[1] - points[3]
         return width * height
-    elif annotation_type in [AnnotationType.polygon.value, AnnotationType.segmentation.value]:
+    elif annotation_type in [
+        AnnotationType.polygon.value,
+        AnnotationType.segmentation.value,
+    ]:
         x = points[0::2]
         y = points[1::2]
-        return 0.5 * np.abs(
-            np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1))
-        )
+        return 0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1)))
     else:
         raise Exception(f"Unsupported annotation type: {annotation_type}")
 
@@ -275,12 +368,17 @@ def __serialize(value: any) -> any:
 # YOLO
 
 
-def to_yolo(tasks: list, classes: list) -> tuple:
+def to_yolo(project_type: str, tasks: list, classes: list, output_dir: str) -> tuple:
     if len(classes) == 0:
-        coco = to_coco(tasks)
+        coco = to_coco(project_type=project_type, tasks=tasks, output_dir=output_dir)
         return __coco2yolo(coco)
     else:
-        return __to_yolo(tasks, classes)
+        return __to_yolo(
+            project_type=project_type,
+            tasks=tasks,
+            classes=classes,
+            output_dir=output_dir,
+        )
 
 
 def __coco2yolo(coco: dict) -> tuple:
@@ -288,8 +386,8 @@ def __coco2yolo(coco: dict) -> tuple:
 
     annos = []
     for image in coco["images"]:
-        dw = 1. / image["width"]
-        dh = 1. / image["height"]
+        dw = 1.0 / image["width"]
+        dh = 1.0 / image["height"]
 
         # Get objects
         objs = []
@@ -322,58 +420,86 @@ def __coco2yolo(coco: dict) -> tuple:
             objs.append(" ".join(obj))
 
         # get annotation
-        anno = {
-            "filename": image["file_name"],
-            "object": objs
-        }
+        anno = {"filename": image["file_name"], "object": objs}
         annos.append(anno)
 
     return annos, categories
 
 
-def __to_yolo(tasks: list, classes: list) -> tuple:
+def __to_yolo(project_type: str, tasks: list, classes: list, output_dir: str) -> tuple:
     annos = []
     for task in tasks:
         if task["height"] == 0 or task["width"] == 0:
             continue
-        objs = []
-        data = [{"annotation": annotation, "task": task, "classes": classes}
-                for annotation in task["annotations"]]
-        with ThreadPoolExecutor(max_workers=8) as executor:
-            results = executor.map(__get_yolo_annotation, data)
-            for result in results:
-                if not result:
-                    continue
-                objs.append(" ".join(result))
-            anno = {
-                "filename": task["name"],
-                "object": objs
-            }
+
+        if is_video_project_type(project_type):
+            image_file_names = _export_image_files_for_video_task(
+                task, str((Path(output_dir) / "images").resolve())
+            )
+
+            def get_annotation_points(anno, index):
+                return _get_annotation_points_for_video_annotation(anno, index)
+
+        else:
+            image_file_names = [task["name"]]
+
+            def get_annotation_points(anno, _):
+                return _get_annotation_points_for_image_annotation(anno)
+
+        for index, image_file_name in enumerate(image_file_names, 1):
+            params = [
+                {
+                    "annotation_value": annotation["value"],
+                    "annotation_type": annotation["type"],
+                    "annotation_points": get_annotation_points(annotation, index),
+                    "width": task["width"],
+                    "height": task["height"],
+                    "classes": classes,
+                }
+                for annotation in task["annotations"]
+            ]
+            with ThreadPoolExecutor(max_workers=8) as executor:
+                image_anno_dicts = executor.map(__get_yolo_annotation, params)
+
+            filtered_image_anno_dicts = list(filter(None, image_anno_dicts))
+
+            anno = {"filename": image_file_name}
+
+            if len(filtered_image_anno_dicts) > 0:
+                anno["object"] = [
+                    " ".join(anno)
+                    for anno in sorted(filtered_image_anno_dicts, key=itemgetter(0))
+                    if anno
+                ]
+
             annos.append(anno)
 
-    categories = map(lambda val: {"name": val}, classes)
+    categories = map(lambda val: {"name": val}, sorted(classes))
 
     return annos, categories
 
 
 def __get_yolo_annotation(data: dict) -> dict:
-    annotation = data["annotation"]
-    points = annotation["points"]
-    annotation_type = annotation["type"]
-    value = annotation["value"]
+    points = data["annotation_points"]
+    annotation_type = data["annotation_type"]
+    value = data["annotation_value"]
     classes = list(data["classes"])
-    task = data["task"]
-    if annotation_type != AnnotationType.bbox.value and annotation_type != AnnotationType.polygon.value:
+    if (
+        annotation_type != AnnotationType.bbox.value
+        and annotation_type != AnnotationType.polygon.value
+    ):
         return None
     if not points or len(points) == 0:
         return None
-    if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
+    if annotation_type == AnnotationType.bbox.value and (
+        int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])
+    ):
         return None
-    if not annotation["value"] in classes:
+    if value not in classes:
         return None
 
-    dw = 1. / task["width"]
-    dh = 1. / task["height"]
+    dw = 1.0 / data["width"]
+    dh = 1.0 / data["height"]
 
     bbox = __to_bbox(annotation_type, points)
     xmin = bbox[0]
@@ -395,83 +521,115 @@ def __get_yolo_annotation(data: dict) -> dict:
 
 
 def _truncate(n, decimals=0) -> float:
-    multiplier = 10 ** decimals
+    multiplier = 10**decimals
     return int(n * multiplier) / multiplier
 
 
 # Pascal VOC
 
 
-def to_pascalvoc(tasks: list) -> list:
+def to_pascalvoc(project_type: str, tasks: list, output_dir: str) -> list:
     pascalvoc = []
     for task in tasks:
         if task["height"] == 0 or task["width"] == 0:
             continue
 
-        pascal_objs = []
-        data = [{"annotation": annotation}
-                for annotation in task["annotations"]]
-        with ThreadPoolExecutor(max_workers=8) as executor:
-            results = executor.map(__get_pascalvoc_obj, data)
+        if is_video_project_type(project_type):
+            image_file_names = _export_image_files_for_video_task(
+                task, str((Path(output_dir) / "images").resolve())
+            )
 
-        for result in results:
-            if not result:
-                continue
-            pascal_objs.append(result)
+            def get_annotation_points(anno, index):
+                return _get_annotation_points_for_video_annotation(anno, index)
 
-        voc = {
-            "annotation": {
-                "filename": task["name"],
-                "size": {
-                    "width": task["width"],
-                    "height": task["height"],
-                    "depth": 3,
-                },
-                "segmented": 0,
-                "object": pascal_objs
+        else:
+            image_file_names = [task["name"]]
+
+            def get_annotation_points(anno, _):
+                return _get_annotation_points_for_image_annotation(anno)
+
+        for index, image_file_name in enumerate(image_file_names, 1):
+            params = [
+                {
+                    "annotation_type": annotation["type"],
+                    "annotation_value": annotation["value"],
+                    "annotation_points": get_annotation_points(annotation, index),
+                    "annotation_attributes": annotation["attributes"],
+                }
+                for annotation in task["annotations"]
+            ]
+
+            with ThreadPoolExecutor(max_workers=8) as executor:
+                pascalvoc_objs = executor.map(__get_pascalvoc_obj, params)
+
+            filtered_pascalvoc_objs = list(filter(None, pascalvoc_objs))
+
+            voc = {
+                "annotation": {
+                    "filename": image_file_name,
+                    "size": {
+                        "width": task["width"],
+                        "height": task["height"],
+                        "depth": 3,
+                    },
+                    "segmented": 0,
+                }
             }
-        }
-        pascalvoc.append(voc)
+
+            if len(filtered_pascalvoc_objs) > 0:
+                voc["annotation"]["object"] = list(
+                    sorted(filtered_pascalvoc_objs, key=itemgetter("name"))
+                )
+
+            pascalvoc.append(voc)
     return pascalvoc
 
 
 def __get_pascalvoc_obj(data: dict) -> dict:
-    annotation = data["annotation"]
-    points = annotation["points"]
-    annotation_type = annotation["type"]
-    if annotation_type != AnnotationType.bbox.value and annotation_type != AnnotationType.polygon.value:
+    points = data["annotation_points"]
+    type = data["annotation_type"]
+    value = data["annotation_value"]
+    attributes = data["annotation_attributes"]
+    if type != AnnotationType.bbox.value and type != AnnotationType.polygon.value:
         return None
     if not points or len(points) == 0:
         return None
-    if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
+    if type == AnnotationType.bbox.value and (
+        int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])
+    ):
         return None
-    bbox = __to_bbox(annotation_type, points)
+    bbox = __to_bbox(type, points)
     x = bbox[0]
     y = bbox[1]
     w = bbox[2]
     h = bbox[3]
 
     return {
-        "name": annotation["value"],
+        "name": value,
         "pose": "Unspecified",
-                "truncated": __get_pascalvoc_tag_value(annotation, "truncated"),
-                "occluded": __get_pascalvoc_tag_value(annotation, "occluded"),
-                "difficult": __get_pascalvoc_tag_value(annotation, "difficult"),
-                "bndbox": {
-                    "xmin": math.floor(x),
-                    "ymin": math.floor(y),
-                    "xmax": math.floor(x + w),
-                    "ymax": math.floor(y + h),
+        "truncated": __get_pascalvoc_tag_value(attributes, "truncated"),
+        "occluded": __get_pascalvoc_tag_value(attributes, "occluded"),
+        "difficult": __get_pascalvoc_tag_value(attributes, "difficult"),
+        "bndbox": {
+            "xmin": math.floor(x),
+            "ymin": math.floor(y),
+            "xmax": math.floor(x + w),
+            "ymax": math.floor(y + h),
         },
     }
 
 
-def __get_pascalvoc_tag_value(annotation: dict, target_tag_name: str) -> int:
-    attributes = annotation["attributes"]
+def __get_pascalvoc_tag_value(attributes: list, target_tag_name: str) -> int:
     if not attributes:
         return 0
     related_attr = next(
-        (attribute for attribute in attributes if attribute["type"] == "switch" and attribute["key"] == target_tag_name), None)
+        (
+            attribute
+            for attribute in attributes
+            if attribute["type"] == "switch" and attribute["key"] == target_tag_name
+        ),
+        None,
+    )
     return int(related_attr["value"]) if related_attr else 0
 
 
@@ -494,7 +652,8 @@ def to_labelme(tasks: list) -> list:
             if annotation["type"] == "segmentation":
                 for i in range(int(len(points[0][0]) / 2)):
                     shape_points.append(
-                        [points[0][0][i * 2], points[0][0][(i * 2) + 1]])
+                        [points[0][0][i * 2], points[0][0][(i * 2) + 1]]
+                    )
             else:
                 for i in range(int(len(points) / 2)):
                     shape_points.append([points[i * 2], points[(i * 2) + 1]])
@@ -504,18 +663,20 @@ def to_labelme(tasks: list) -> list:
                 "points": shape_points,
                 "group_id": None,
                 "shape_type": shape_type,
-                "flags": {}
+                "flags": {},
             }
             shapes.append(shape)
-        labelmes.append({
-            "version": "4.5.9",
-            "flags": {},
-            "shapes": shapes,
-            "imagePath": task["name"],
-            "imageData": None,
-            "imageHeight": task["height"],
-            "imageWidth": task["width"],
-        })
+        labelmes.append(
+            {
+                "version": "4.5.9",
+                "flags": {},
+                "shapes": shapes,
+                "imagePath": task["name"],
+                "imageData": None,
+                "imageHeight": task["height"],
+                "imageWidth": task["width"],
+            }
+        )
     return labelmes
 
 
@@ -559,10 +720,14 @@ def to_pixel_coordinates(tasks: list) -> list:
                 xmax = max([points[0], points[2]])
                 ymax = max([points[1], points[3]])
                 annotation["points"] = [
-                    xmin, ymin,
-                    xmax, ymin,
-                    xmax, ymax,
-                    xmin, ymax,
+                    xmin,
+                    ymin,
+                    xmax,
+                    ymin,
+                    xmax,
+                    ymax,
+                    xmin,
+                    ymax,
                 ]
             else:
                 continue
@@ -580,8 +745,7 @@ def to_pixel_coordinates(tasks: list) -> list:
                     new_regions.append(new_region)
                 annotation["points"] = new_regions
             elif annotation["type"] == AnnotationType.polygon.value:
-                new_points = __remove_duplicated_coordinates(
-                    annotation["points"])
+                new_points = __remove_duplicated_coordinates(annotation["points"])
                 annotation["points"] = new_points
     return tasks
 
@@ -596,29 +760,31 @@ def __remove_duplicated_coordinates(points: List[int]) -> List[int]:
     new_points = []
     for i in range(int(len(points) / 2)):
         if i == 0:
-            new_points.append(points[i*2])
-            new_points.append(points[i*2 + 1])
+            new_points.append(points[i * 2])
+            new_points.append(points[i * 2 + 1])
 
-        if new_points[-2] == points[i*2] and new_points[-1] == points[i*2 + 1]:
+        if new_points[-2] == points[i * 2] and new_points[-1] == points[i * 2 + 1]:
             continue
 
         if len(new_points) <= 2:
-            new_points.append(points[i*2])
-            new_points.append(points[i*2 + 1])
+            new_points.append(points[i * 2])
+            new_points.append(points[i * 2 + 1])
         else:
-            if new_points[-4] == new_points[-2] and new_points[-2] == points[i*2]:
+            if new_points[-4] == new_points[-2] and new_points[-2] == points[i * 2]:
                 new_points.pop()
                 new_points.pop()
-                new_points.append(points[i*2])
-                new_points.append(points[i*2 + 1])
-            elif new_points[-3] == new_points[-1] and new_points[-1] == points[i*2 + 1]:
+                new_points.append(points[i * 2])
+                new_points.append(points[i * 2 + 1])
+            elif (
+                new_points[-3] == new_points[-1] and new_points[-1] == points[i * 2 + 1]
+            ):
                 new_points.pop()
                 new_points.pop()
-                new_points.append(points[i*2])
-                new_points.append(points[i*2 + 1])
+                new_points.append(points[i * 2])
+                new_points.append(points[i * 2 + 1])
             else:
-                new_points.append(points[i*2])
-                new_points.append(points[i*2 + 1])
+                new_points.append(points[i * 2])
+                new_points.append(points[i * 2 + 1])
     return new_points
 
 
@@ -636,8 +802,8 @@ def __get_pixel_coordinates(points: List[int or float]) -> List[int]:
         if i == 0:
             continue
 
-        prev_x = int(points[(i-1) * 2])
-        prev_y = int(points[(i-1) * 2 + 1])
+        prev_x = int(points[(i - 1) * 2])
+        prev_y = int(points[(i - 1) * 2 + 1])
         x = int(points[i * 2])
         y = int(points[i * 2 + 1])
 
@@ -658,13 +824,13 @@ def __get_pixel_coordinates(points: List[int or float]) -> List[int]:
     return new_points
 
 
-def execute_coco_to_fastlabel(coco: dict ,annotation_type:str) -> dict:
+def execute_coco_to_fastlabel(coco: dict, annotation_type: str) -> dict:
     coco_images = {}
     for c in coco["images"]:
         coco_images[c["id"]] = c["file_name"]
 
     coco_categories = {}
-    coco_categories_keypoints={}
+    coco_categories_keypoints = {}
     for c in coco["categories"]:
         coco_categories[c["id"]] = c["supercategory"]
         coco_categories_keypoints[c["id"]] = c["keypoints"]
@@ -686,7 +852,10 @@ def execute_coco_to_fastlabel(coco: dict ,annotation_type:str) -> dict:
             if not category_name:
                 return
 
-            if annotation_type in [AnnotationType.bbox.value, AnnotationType.polygon.value]:
+            if annotation_type in [
+                AnnotationType.bbox.value,
+                AnnotationType.polygon.value,
+            ]:
                 segmentation = target_coco_annotation["segmentation"][0]
                 annotation_type = ""
                 if len(segmentation) == 4:
@@ -703,32 +872,37 @@ def execute_coco_to_fastlabel(coco: dict ,annotation_type:str) -> dict:
             elif annotation_type == AnnotationType.pose_estimation.value:
                 keypoints = []
                 target_coco_annotation_keypoints = target_coco_annotation["keypoints"]
-                keypoint_keys = coco_categories_keypoints[target_coco_annotation["category_id"]]
+                keypoint_keys = coco_categories_keypoints[
+                    target_coco_annotation["category_id"]
+                ]
                 # coco keypoint style [100,200,1,300,400,1,500,600,2] convert to [[100,200,1],[300,400,1],[500,600,2]]
-                keypoint_values = [target_coco_annotation_keypoints[i:i + 3] for i in range(0, len(target_coco_annotation_keypoints), 3)]
+                keypoint_values = [
+                    target_coco_annotation_keypoints[i : i + 3]
+                    for i in range(0, len(target_coco_annotation_keypoints), 3)
+                ]
                 for index, keypoint_key in enumerate(keypoint_keys):
                     keypoint_value = keypoint_values[index]
                     if keypoint_value[2] == 0:
                         continue
-                    if not keypoint_value[2] in [1, 2]: 
-                        raise FastLabelInvalidException(f"Visibility flag must be 0 or 1, 2 . annotation_id: {target_coco_annotation['id']}", 422)
-                    # fastlabel occulusion is 0 or 1 . coco occulusion is 1 or 2. 
+                    if not keypoint_value[2] in [1, 2]:
+                        raise FastLabelInvalidException(
+                            f"Visibility flag must be 0 or 1, 2 . annotation_id: {target_coco_annotation['id']}",
+                            422,
+                        )
+                    # fastlabel occulusion is 0 or 1 . coco occulusion is 1 or 2.
                     keypoint_value[2] = keypoint_value[2] - 1
-                    keypoints.append({
-                        "key": keypoint_key,
-                        "value": keypoint_value
-                    })
+                    keypoints.append({"key": keypoint_key, "value": keypoint_value})
 
                 annotations.append(
                     {
                         "value": category_name,
                         "type": annotation_type,
-                        "keypoints": keypoints
+                        "keypoints": keypoints,
                     }
                 )
             else:
                 raise FastLabelInvalidException(
-                "Annotation type must be bbox or polygon ,pose_estimation.", 422
+                    "Annotation type must be bbox or polygon ,pose_estimation.", 422
                 )
 
         results[coco_images[coco_image_key]] = annotations
@@ -817,24 +991,16 @@ def execute_yolo_to_fastlabel(
 
             classs_name = classes[str(yolo_class_id)]
 
-            yolo_center_x_point = float(
-                image_width) * float(yolo_center_x_ratio)
-            yolo_center_y_point = float(
-                image_height) * float(yolo_center_y_ratio)
-            yolo_anno_width_size = float(
-                image_width) * float(yolo_anno_width_ratio)
-            yolo_anno_height_size = float(
-                image_height) * float(yolo_anno_height_ratio)
+            yolo_center_x_point = float(image_width) * float(yolo_center_x_ratio)
+            yolo_center_y_point = float(image_height) * float(yolo_center_y_ratio)
+            yolo_anno_width_size = float(image_width) * float(yolo_anno_width_ratio)
+            yolo_anno_height_size = float(image_height) * float(yolo_anno_height_ratio)
 
             points = []
-            points.append(yolo_center_x_point -
-                          (yolo_anno_width_size / 2))  # x1
-            points.append(yolo_center_y_point -
-                          (yolo_anno_height_size / 2))  # y1
-            points.append(yolo_center_x_point +
-                          (yolo_anno_width_size / 2))  # x2
-            points.append(yolo_center_y_point +
-                          (yolo_anno_height_size / 2))  # y2
+            points.append(yolo_center_x_point - (yolo_anno_width_size / 2))  # x1
+            points.append(yolo_center_y_point - (yolo_anno_height_size / 2))  # y1
+            points.append(yolo_center_x_point + (yolo_anno_width_size / 2))  # x2
+            points.append(yolo_center_y_point + (yolo_anno_height_size / 2))  # y2
             annotations.append(
                 {
                     "value": classs_name,
@@ -865,3 +1031,79 @@ def __get_annotation_type_by_labelme(shape_type: str) -> str:
     if shape_type == "line":
         return "line"
     return None
+
+
+@contextmanager
+def VideoCapture(*args, **kwds):
+    videoCapture = cv2.VideoCapture(*args, **kwds)
+    try:
+        yield videoCapture
+    finally:
+        videoCapture.release()
+
+
+def _download_file(url: str, output_file_path: str, chunk_size: int = 8192) -> str:
+    with requests.get(url, stream=True) as stream:
+        stream.raise_for_status()
+        with open(file=output_file_path, mode="wb") as file:
+            for chunk in stream.iter_content(chunk_size=chunk_size):
+                if chunk:
+                    file.write(chunk)
+            return file.name
+
+
+def _export_image_files_for_video_file(
+    file_path: str,
+    output_dir_path: str,
+    basename: str,
+):
+    image_file_names = []
+    with VideoCapture(file_path) as cap:
+        if not cap.isOpened():
+            raise FastLabelInvalidException(
+                (
+                    "Video to image conversion failed. Video could not be opened.",
+                    " Download may have failed or there is a problem with the video.",
+                ),
+                422,
+            )
+        digit = len(str(int(cap.get(cv2.CAP_PROP_FRAME_COUNT))))
+        frame_num = 0
+        while True:
+            ret, frame = cap.read()
+            if not ret:
+                break
+
+            image_file_name = f"{basename}_{str(frame_num).zfill(digit)}.jpg"
+            image_file_path = os.path.join(output_dir_path, image_file_name)
+            os.makedirs(output_dir_path, exist_ok=True)
+            cv2.imwrite(image_file_path, frame)
+            frame_num += 1
+            image_file_names.append(image_file_name)
+    return image_file_names
+
+
+def _export_image_files_for_video_task(video_task: dict, output_dir_path: str):
+    with NamedTemporaryFile(prefix="fastlabel-sdk-") as video_file:
+        video_file_path = _download_file(
+            url=video_task["url"], output_file_path=video_file.name
+        )
+        return _export_image_files_for_video_file(
+            file_path=video_file_path,
+            output_dir_path=output_dir_path,
+            basename=Path(video_task["name"]).stem,
+        )
+
+
+def _get_annotation_points_for_video_annotation(annotation: dict, index: int):
+    points = annotation.get("points")
+    if not points:
+        return None
+    video_point_datum = points.get(str(index))
+    if not video_point_datum:
+        return None
+    return video_point_datum["value"]
+
+
+def _get_annotation_points_for_image_annotation(annotation: dict):
+    return annotation.get("points")
