@@ -30,10 +30,20 @@ def is_audio_supported_ext(file_path: str) -> bool:
     return file_path.lower().endswith((".mp3", ".wav", ".m4a"))
 
 
+def is_pcd_supported_ext(file_path: str) -> bool:
+    # .ply is not yet supported. To support it, modification of the API
+    # needs to be considered as well.
+    return file_path.lower().endswith((".pcd"))
+
+
 def is_image_supported_size(file_path: str) -> bool:
     return os.path.getsize(file_path) <= const.SUPPORTED_IMAGE_SIZE
 
 
+def is_image_supported_size_for_inference(file_path: str) -> bool:
+    return os.path.getsize(file_path) <= const.SUPPORTED_INFERENCE_IMAGE_SIZE
+
+
 def is_video_supported_size(file_path: str) -> bool:
     return os.path.getsize(file_path) <= const.SUPPORTED_VIDEO_SIZE
 
@@ -46,6 +56,14 @@ def is_audio_supported_size(file_path: str) -> bool:
     return os.path.getsize(file_path) <= const.SUPPORTED_AUDIO_SIZE
 
 
+def is_pcd_supported_size(file_path: str) -> bool:
+    return os.path.getsize(file_path) <= const.SUPPORTED_PCD_SIZE
+
+
+def is_video_project_type(project_type: str):
+    return type(project_type) is str and project_type.startswith("video_")
+
+
 def is_json_ext(file_name: str) -> bool:
     return file_name.lower().endswith(".json")
 
@@ -75,6 +93,32 @@ def reverse_points(points: List[int]) -> List[int]:
     return reversed_points
 
 
+def sort_segmentation_points(points: List[int]) -> List[int]:
+    """
+    e.g.)
+    [1, 2, 1, 1, 2, 1, 2, 2, 1, 2] => [1, 1, 2, 1, 2, 2, 1, 2, 1, 1]
+    """
+    points_array = np.array(points).reshape((-1, 2))[1:]
+    base_point_index = 0
+    points_list = points_array.tolist()
+    for index, val in enumerate(points_list):
+        if index == 0:
+            continue
+        if (
+            val[1] <= points_list[base_point_index][1]
+            and val[0] <= points_list[base_point_index][0]
+        ):
+            base_point_index = index
+    new_points_array = np.vstack(
+        [
+            points_array[base_point_index:],
+            points_array[:base_point_index],
+            np.array([points_array[base_point_index]]),
+        ]
+    )
+    return new_points_array.ravel().tolist()
+
+
 def is_clockwise(points: list) -> bool:
     """
     points: [x1, y1, x2, y2, x3, y3, ... xn, yn]
